import logging
from hc_upgrade_tools import utility as util
import os


# get logger for this file
_logger = util.setup_logging(
    "download_pkg", "hc_upgrade_tools.download.log", logging.DEBUG
)

# init sub command parser as subparser of skeleton.py
# subparser add_argument in this file
def init_subparser(subparsers):
    """Init sub command parser as subparser of skeleton.py

    Args:
      subparsers (List[str]): command line parameters as list of strings
          (for example  ``["--help"]``).

    Returns:
      :obj:`argparse.Namespace`: command line parameters namespace
    """
    subparser = subparsers.add_parser("dl-pkg", help="sub command help")

    # add argument "--host"
    subparser.add_argument(
        "--host", help="host name or ip address", type=str, metavar="STR", required=True
    )
    # add argument "--port"
    subparser.add_argument(
        "--port", help="port number", type=int, metavar="INT", required=True
    )
    # add argument resource-path
    subparser.add_argument(
        "--resource-path",
        help="resource path",
        type=str,
        metavar="STR",
        default="/api/githook/request-artifacts",
    )
    # add argument "projectID"
    subparser.add_argument("--projectID", help="project ID", type=str, metavar="STR", required=True)
    # add argument "project-dir"
    subparser.add_argument(
        "--project-dir",
        help="项目跟目录",
        type=str,
        metavar="STR",
        required=True,
    )
    # add argument "release-root-dir"
    subparser.add_argument(
        "--release-root-dir",
        help="releases 默认在项目根目录下的releases目录, 也可以通过此参数指定",
        type=str,
        metavar="STR",
    )
    # add argument "target-source-sub-folder-in-artifact"
    subparser.add_argument(
        "--target-sub-folder-in-artifact",
        help="交付物解压后的目标文件件",
        type=str,
        metavar="STR",
    )
    # add list argument "extra-links"
    subparser.add_argument(
        "--extra-symbolics",
        help="额外链接, 相对于项目的根目录。例如：--extra-symbolics 'current/dock-compose.yml:shared/dock-compose.yml', 'current/config:shared/config' \
            形式为： symbolic_link_path:source_path",
        type=str,
        metavar="STR",
        nargs="*",
        default=[],
    )

    subparser.add_argument('--restart-docker', help='restart docker after handle update artifacts', action='store_true')

    subparser.set_defaults(func=main_process)

    return subparser


def main_process(args):
    upgrade_porcess_root_dir = "/tmp/hc_upgrade_tools"

    project_id = args.projectID
    original_artifacts_download_dir = os.path.join(upgrade_porcess_root_dir, "artifacts")
    artifact_name = "artifact.zip"

    artifacts_path = util.download_artifact(
        args.host, args.port, args.resource_path, project_id, original_artifacts_download_dir, artifact_name
    )
    _logger.debug(f"artifacts_path: {artifacts_path}")
    _logger.debug(f"upgrade_porcess_root_dir: {upgrade_porcess_root_dir}")

    if artifacts_path:
        # releases 默认在项目根目录下的releases目录
        project_release_dir = os.path.join(args.project_dir, "releases")

        if args.release_root_dir:
            project_release_dir = args.release_root_dir

        # 生成目标文件，以日期为文件名
        target_release_dir = util.create_new_release_dir(project_release_dir)
        _logger.debug(f"target_release_dir: {target_release_dir}")

        util.handle_artifacts(
            artifacts_path,
            target_release_dir,
            source_sub_folder=args.target_sub_folder_in_artifact,
        )

        # 将目标文件夹软连接到current目录
        current_link = os.path.join(args.project_dir, "current")
        util.link_current_to_new_release(current_link, target_release_dir)


        # 创建其他额外的链接
        _logger.debug(f"extra_symbolics: {args.extra_symbolics}")
        util.handle_extra_links(args)

        util.restart_docker_container(args)



import pytest


__author__ = "shixukai"
__copyright__ = "shixukai"
__license__ = "MIT"


def test_fib():
    """API Tests"""
    assert sum([1, 2, 3]) == 6, "Should be 6"

def test_main(capsys):
    """CLI Tests"""
    assert sum([1, 2, 3]) == 6, "Should be 6"



import pytest
import unittest.mock

import hc_upgrade_tools.utility as util

# test util.handle_artifacts
def test_handle_artifacts():
    # test handle_artifacts with source_sub_folder is None
    with unittest.mock.patch('shutil.rmtree') as mock_rmtree:
        with unittest.mock.patch('os.path.exists') as mock_exists:
            with unittest.mock.patch('os.makedirs') as mock_makedirs:
                with unittest.mock.patch('zipfile.ZipFile') as mock_zipfile:
                    # mock logger, return a mock object
                    mock_logger = unittest.mock.Mock()
                    util.get_logger = unittest.mock.Mock(return_value=mock_logger)

                    mock_zipfile.return_value.__enter__.return_value.extractall = unittest.mock.Mock()

                    mock_exists.return_value = True

                    util.handle_artifacts("source_file_path", "target_dir")

                    mock_makedirs.assert_not_called()
                    mock_rmtree.assert_called_once_with("target_dir")

                    assert mock_exists.call_count == 2
                    mock_zipfile.return_value.__enter__.return_value.extractall.assert_called_once_with("target_dir")


    # test handle_artifacts with source_sub_folder is not None
    with unittest.mock.patch('shutil.rmtree') as mock_rmtree:
        with unittest.mock.patch('os.path.exists') as mock_exists:
            with unittest.mock.patch('os.makedirs') as mock_makedirs:
                with unittest.mock.patch('os.makedirs') as mock_oslistdir:
                    #mock zip file with
                    with unittest.mock.patch('zipfile.ZipFile') as mock_zipfile:
                        # mock logger, return a mock object
                        mock_logger = unittest.mock.Mock()
                        util.get_logger = unittest.mock.Mock(return_value=mock_logger)

                        mock_oslistdir.return_value = []
                        mock_exists.return_value = True
                        mock_zipfile.return_value.__enter__.return_value.extractall = unittest.mock.Mock()
                        util.handle_artifacts("source_file_path", "target_dir", "source_sub_folder")

                        mock_rmtree.assert_called_once_with("target_dir")

                        assert mock_exists.call_count == 3
                        mock_makedirs.assert_not_called()

                        mock_zipfile.assert_called_once_with("source_file_path")
                        mock_zipfile.return_value.__enter__.return_value.extractall.assert_called_once_with("target_dir/tmp")



from ast import Store
import os
import argparse


if __name__=="__main__":
    # add arg parse
    parser = argparse.ArgumentParser(description="升级脚本使用说明")
    parser.add_argument('--update-middle-version', help='increse middle version', action='store_true')
    parser.add_argument('--update-major-version', help='increse big version', action='store_true')
    args = parser.parse_args()


    # check if current git work tree is clean and has no untracked files
    # if yes, checkout to release branch else exit
    if not os.system("git diff --quiet && git diff --cached --quiet"):
        # if release and dev on the same commit, exit
        if os.system("git rev-parse --abbrev-ref HEAD") == "release":
            print("release branch is on the same commit as dev branch, exit")
            exit(1)

        # checkout to release branch
        os.system("git checkout release")
        # merge dev branch to release branch
        os.system("git merge dev")
        # if merge success, push release branch to remote
        if not os.system("git diff --quiet && git diff --cached --quiet"):
            # read the latest tag
            latest_tag = os.popen("git describe --abbrev=0 --tags").read()
            # get the latest tag version
            latest_tag_version = latest_tag.split("v")[1]
            # increase the latest tag version
            latest_tag_version = latest_tag_version.split(".")
            # if update major version
            if args.update_major_version:
                latest_tag_version[0] = str(int(latest_tag_version[0]) + 1)
                latest_tag_version[1] = "0"
                latest_tag_version[2] = "0"
            # if update middle version
            elif args.update_middle_version:
                latest_tag_version[1] = str(int(latest_tag_version[1]) + 1)
                latest_tag_version[2] = "0"
            # if update minor version
            else:
                latest_tag_version[2] = str(int(latest_tag_version[2]) + 1)
            # join the latest tag version
            latest_tag_version = ".".join(latest_tag_version)
            # create new tag
            os.system(f"git tag v{latest_tag_version}")
            # push tag to remote
            os.system(f"git push origin v{latest_tag_version}")
            # push release branch to remote
            os.system("git push origin release")
            # run tox -e publish
            os.system("python3 -m tox -e build && python3 -m tox -e publish -- --repository pypi")

            if not os.system("git diff --quiet && git diff --cached --quiet"):
                os.system("git checkout dev")

        else:
            # abort merge
            os.system("git merge --abort")

    else:
        # exit
        print("git work tree is not clean, please commit and push your changes first")
        exit(1)
